# requires that graphics.py be in the same directory
from graphics import *

win = GraphWin("Tic-Tac-Toe", 240, 240)
win.setBackground("white")
win.setCoords(0.0, 0.0, 3.0, 3.0)

board = [["", "", ""],
         ["", "", ""],
         ["", "", ""]]

# check row i for a winner
def check_row(board, i):
    return board[i][0] == board[i][1] == board[i][2] and len(board[i][0]) > 0

# check column i for a winner
def check_column(board, i):
    return board[0][i] == board[1][i] == board[2][i] and len(board[0][i]) > 0

# check each row, column, and diagonal for a winner
# return "X" if X wins, "O" if O wins, and "" if there is no winner yet
def check_winner(board):
    for i in range(len(board)):
        if check_row(board, i):
            return board[i][0]
        if check_column(board, i):
            return board[0][i]
    if board[0][0] == board[1][1] == board[2][2] and len(board[0][0]) > 0:
        return board[0][0]
    if board[0][2] == board[1][1] == board[2][0] and len(board[0][2]) > 0:
        return board[0][2]
    return ""

# determines if the point move is valid (i.e., is that board location emtpy)
def valid_move(move, board):
    x = int(move.getX())
    y = int(move.getY())
    return board[y][x] == ""

# draw the current board
def draw_board(win, board):
    # draw the grid lines
    for i in range(len(board)):
        Line(Point(i, 0), Point(i, 3)).draw(win)
        Line(Point(0, i), Point(3, i)).draw(win)
    # draw the Xs and Os
    for row in range(len(board)):
        for column in range(len(board)):
            cell = Text(Point(column + 0.5, row + 0.5), board[row][column])
            cell.setSize(36)
            cell.draw(win)

draw_board(win, board)
moves_so_far = 0
winner = check_winner(board)
# game loop
while winner == "" and moves_so_far < 9 and win.isOpen():
    # get the location of Xs click
    move = win.getMouse()
    while not valid_move(move, board):
        move = win.getMouse()
    # the location of the click is a float (decimal number)
    # but we want to use them as indexes for board
    # so we convert them to integers
    x = int(move.getX())
    y = int(move.getY())
    # set Xs move, add one to the moves_so_far, and draw the board
    board[y][x] = "X"
    moves_so_far += 1
    draw_board(win, board)
    winner = check_winner(board)
    if winner == "": # if X didn't win, do Os turn
        # get the location of the Os click
        move = win.getMouse()
        while not valid_move(move, board):
            move = win.getMouse()
        x = int(move.getX())
        y = int(move.getY())
        # set Os move, add one to the moves_so_far, and draw the board
        board[y][x] = "O"
        moves_so_far += 1
        draw_board(win, board)
        winner = check_winner(board)

if win.isOpen():
    if winner == "": # if we have no winner
        winner = "No one"
    # display a message for the winner
    msg = Text(Point(1.5, 2), winner + " wins!!")
    msg.setSize(36)
    msg.draw(win)
    # always end with win.getMouse() to have the window wait for a click
    # instead of immediately closing
    win.getMouse()
